[data-md-color-scheme="chrkong"] {
    --md-primary-fg-color: #40b569;
    --md-primary-bg-color: #e8e8e8ff;
    --md-default-bg-color: #2e2e2eff;
    --md-typeset-color: #e8e8e8ff;
    --md-default-fg-color--light: #e8e8e8ff;
    --md-footer-bg-color: #2e2e2eff;
    --md-typeset-a-color: #40b569;
    --md-accent-fg-color: #40b569;
}